#version 310 es 
                
precision highp float; 
                       
                       
uniform vec4 iMouse;      
uniform vec4 iDate;       
uniform float iTime;      
uniform float iFrame;     
                                 
uniform vec3 iResolution; 
uniform vec3 iChannelResolution[4]; 
                                           
uniform sampler2D iChannel0; 
uniform sampler2D iChannel1; 
uniform sampler2D iChannel2; 
uniform sampler2D iChannel3; 
                                              
out vec4 finalColor;                          
                                              
#define MAX_STEPS 100
#define EPSILON 0.005
#define N_EPSILON 0.05
#define PI 3.14159265358979323846

#define ez vec3(0.0, 0.0, 1.0)

mat2 rot(float a) {
    float c = cos(a); float s = sin(a);
    return mat2(c, s, -s, c);
}


float blade(vec3 p, float a){
    
    p.xy *= rot(a); 
    p.xz *= rot(-clamp(0.3 + 0.03*p.y*p.y, 0.3, 0.6));
    
    float psy = p.y - 3.0;
    float ppy = p.y - 1.0;
    
    
    float xs = clamp(0.4 + -0.04*psy*psy, -1.5, 1.0);
    float xp = clamp( 2.0 + 0.02*ppy*ppy,  1.0, 2.5);
    
    
    return length(p - vec3(clamp(p.x, -2.0*xs, xp),
                           clamp(p.y, -3.0, 3.0 - 0.1*p.x*p.x),
                           0.0
                           )) - 0.1;
}

float centre(vec3 p){
    float rad = 2.5;
    float len = 0.7;
    vec2 d = abs(vec2(length(p.xy/rad), p.z/len) ) - 1.0 ;
  	return min(max(d.x,d.y),0.0) + length(max(d,0.0));
}


float concav(vec3 p){
    float rad = 2.3;
    float len = 0.7;
    vec2 d = abs(vec2(length(p.xy/rad), p.z/len) ) - 1.0 ;
  	return min(max(d.x,d.y),0.0) + length(max(d,0.0));
}


float d_sphere(vec3 p, vec3 pos, float r){
    return length(p - pos) - r;
}

float sceneSDF(vec3 p){
    
    p.z += 10.0;
    
    p.xz *= rot(iTime);
    p.xy *= rot(2.6*iTime);
    
    float a = 2.0*PI/7.0;
    mat2 R = rot(a);
 
    vec2 O1 = vec2(0.0, -3.5);
    vec2 O2 = R*O1;
    vec2 O3 = R*O2;
    vec2 O4 = R*O3;
    vec2 O5 = R*O4;
    vec2 O6 = R*O5;
    vec2 O7 = R*O6;
    
    
    float z_pos = 0.0;
    
    float d = min( 
        		   min( blade(p + vec3(O1, z_pos), 0.0*a),
                        blade(p + vec3(O2, z_pos), 1.0*a)) , 
                   min( blade(p + vec3(O3, z_pos), 2.0*a),
                        blade(p + vec3(O4, z_pos), 3.0*a)));
    
    	  d = min( min( d,
                       	blade(p + vec3(O5, z_pos), 4.0*a)),
                   min( blade(p + vec3(O6, z_pos), 5.0*a),
           				blade(p + vec3(O7, z_pos), 6.0*a)));
                       
    d =  min(d,centre(p + vec3(0.0, 0.0, 0.7 + z_pos)));
    
    d = max(d, -concav(p + vec3(0.0, 0.0, 1.0 + z_pos)));
    
    vec2 p1 = vec2(0.2, 0.2);
    vec2 p2 = rot(2.0*PI/3.0)*p1;
    vec2 p3 = rot(2.0*PI/3.0)*p2;
    
    float d_logo = min(
        				min( length(p + vec3(p1, 0.0 + z_pos)) - 0.2,
                             length(p + vec3(p2, 0.0 + z_pos)) - 0.2),
                             length(p + vec3(p3, 0.0 + z_pos)) - 0.2);
    
    d = max(d, - d_logo);
    
    return d;
    
}




vec3 estimateNormal(vec3 p) {
    return normalize(vec3(
        sceneSDF(vec3(p.x + N_EPSILON, p.y, p.z)) - sceneSDF(vec3(p.x - N_EPSILON, p.y, p.z)),
        sceneSDF(vec3(p.x, p.y + N_EPSILON, p.z)) - sceneSDF(vec3(p.x, p.y - N_EPSILON, p.z)),
        sceneSDF(vec3(p.x, p.y, p.z  + N_EPSILON)) - sceneSDF(vec3(p.x, p.y, p.z - N_EPSILON))
    ));
}

void mainImage( out vec4 fragColor, in vec2 fragCoord ){
    
	vec2 uv = fragCoord.xy - 0.5*iResolution.xy;
    uv *= 2.0/iResolution.y;
    
    uv *= 4.0;
    
    float fov = PI/20.0;
    vec3 cam_pos = vec3(0.0, 0.0, 1.0*(1.0 + 1.0/tan(fov)));
    
    vec3 ray = cam_pos;
    vec3 dir = normalize(vec3(uv.xy, 0.0) - cam_pos);
    
    vec3 light = normalize(vec3(0.5, 0.5, 1.0));
    
    float shade = 0.0;
    float shade_1 = 0.0;
    for (int i = 0; i < MAX_STEPS; i++){
        
        float dist = sceneSDF(ray);
        
        if ( dist < EPSILON ) {
            
            vec3 n = estimateNormal(ray);
            float soft = dot(light,n);
            float soft_s = soft - 1.0;
            shade += 0.8*soft;
            shade += clamp(exp(-soft*soft*10.0), 0.0, 1.0);
            shade += clamp(exp(-soft_s*soft_s*10000.0), 0.0, 1.0);
            
            break;
        }
        else if ( length(ray) > 30.0 )
            break;
        ray += 0.71*dir*dist;
    }
    
    
    
	fragColor = vec4(1.0, 0.9, 0.0, 0.0)*shade;

}

void main(void) 
{               
   vec4 color;                        
   mainImage(color, gl_FragCoord.xy); 
   finalColor = color;                
}
